pro build_vgroup_menu, state, vgn, desc, flag
  HDF_VG_GETINFO, vgn, nentries=n, name=name
  HDF_VG_GETTRS, vgn, tags, refs
  if (state.view_mode eq 0) then $
    select=where(tags eq 1965, n) else $
    select=where((tags eq 720) or (tags eq 1965), n)
  if (n eq 0) then flag=flag-1
  desc = [desc, {CW_PD_MENU_S, flags:flag, name:name}]
  for i=0, n-1 do begin
    ; If this is the last item in this Vgroup set flag=3
    if (i eq n-1) then flag=3 else flag=1
    j = select(i)
    case tags(j) of
      720: begin
        index = hdf_sd_reftoindex(state.sdsid, refs(j))
        sds_id = hdf_sd_select(state.sdsid, index)
        build_sds_menu, state, sds_id, desc, flag
      end
      1965: begin
        t = HDF_VG_ATTACH(state.fileid, refs(j))
        build_vgroup_menu, state, t, desc, flag
      end
    endcase
  endfor
end


pro build_sds_menu, state, sds_id, desc, flag
  HDF_SD_GETINFO, sds_id, name=name, dims=dims, type=type, natts=natts
  if ((type eq 'STRING') and (n_elements(dims) eq 1)) then begin
    HDF_SD_GETDATA, sds_id, data
    str = name + '  ' + ' = "' + data(0) + '"'
  endif else if ((n_elements(dims) eq 1) and (dims(0) eq 1)) then begin
    HDF_SD_GETDATA, sds_id, data
    str = name + '  ' + ' = ' + strtrim(string(data(0)),2)
  endif else begin
    count = dims*0 + 1 & count(0)=2
    HDF_SD_GETDATA, sds_id, data, count=count
    str = name + '  ' + type + '( '
    for i=0, n_elements(dims)-1 do str = str+strtrim(dims(i),2)+' '
    str = str + ')'
    str = str + ' = ' + strtrim(string(data(0)),2) + ', ' + $
                        strtrim(string(data(1)),2) + ', ...'
  endelse
  if (state.view_mode lt 2) then natts = 0  ; Don't display attributes
  if (natts eq 0) then flag=flag-1
  desc = [desc, {CW_PD_MENU_S, flags:flag, name:str}]
  if (natts gt 0) then begin
    for j=0,natts-1 do begin
      HDF_SD_ATTRINFO,sds_id,j,name=name,data=d,count=count,type=type
      if (type eq 'STRING') then begin
        d = '"' + d + '"'
      endif
      str = name+' = '+string(d)
      desc = [desc, {CW_PD_MENU_S, flags:0, name:str(0)}]
    endfor
    desc(n_elements(desc)-1).flags=2
  endif
  HDF_SD_ENDACCESS,sds_id
end

pro build_globatts_menu, state, desc
  hdf_sd_fileinfo,state.sdsid,nmfsds,nglobatts
  if (nglobatts gt 0) then begin
    desc = [desc, {CW_PD_MENU_S, flags:1, name:'Global attributes'}]
    for j=0,nglobatts-1 do begin
      HDF_SD_ATTRINFO,state.sdsid,j,name=name,data=d,count=count,type=type
      if (type eq 'STRING') then begin
        d = '"' + d + '"'
      endif
      str = name+' = '+string(d)
      desc = [desc, {CW_PD_MENU_S, flags:0, name:str(0)}]
    endfor
    desc(n_elements(desc)-1).flags=2
  endif
end


pro aps_build_menu, state, desc
; Loop on Vgroups of class APS_entry
desc = {CW_PD_MENU_S, flags:1, name:'Browse ...'}
if (state.fileid eq -1) then return
numvg=hdf_number(state.fileid,tag=1965)
vgid = HDF_VG_GETID(state.fileid, -1)
build_globatts_menu, state, desc
for i=0, numvg-1 do begin
  vgn = HDF_VG_ATTACH(state.fileid, vgid)
  HDF_VG_GETINFO, vgn, class=class, name=name
  if  (class eq 'APS_entry') or (class eq 'APS_default') then begin
    build_vgroup_menu, state, vgn, desc, 1
  endif
  vgid = HDF_VG_GETID(state.fileid, vgid)
endfor
end


pro aps_open_file, state
  ;   Check that it is HDF
if (state.filename ne "") then begin
  if hdf_ishdf(state.filename) ne 1 then begin
    status = widget_message(/ERROR, $
       "File "+state.filename+" is not an HDF file.")
    state.fileid=-1
    state.sdsid=-1
   endif else begin
     state.fileid=hdf_open(state.filename,/read)
     state.sdsid=hdf_sd_start(state.filename,/read)
   endelse
endif else begin
    state.fileid=-1
    state.sdsid=-1
endelse
end

pro new_menu, state
    aps_open_file, state
    aps_build_menu, state, menu
    ;for i=0, n_elements(menu)-1 do print, menu(i)
    state.widget.view = $
        cw_pdmenu(state.widget.view_parent, menu, /return_index)
    if (state.fileid eq -1) then $
        widget_control, state.widget.view, sensitive=0 else $
        widget_control, state.widget.view, sensitive=1
end

pro aps_hdf_browser_event, event

widget_control, event.top, get_uvalue=state, /no_copy

case event.id of

    state.widget.open: begin
        state.filename = pickfile()
        widget_control, event.top, update=0, /hourglass
        widget_control, state.widget.view, /destroy
        new_menu, state
        widget_control, event.top, update=1
    end

    state.widget.exit: begin
        widget_control, event.top, /destroy
        return
    end
   
    state.widget.view_mode: begin
        ; Don't do anything if a new selection was not made
        if (state.view_mode ne event.index) then begin
           state.view_mode = event.index
            widget_control, event.top, update=0, /hourglass
            widget_control, state.widget.view, /destroy
            new_menu, state
            widget_control, event.top, update=1
        endif
    end

    else: begin
        ; This must be a pull-down menu event
        ; print, 'Event value = ', event.value
    end
endcase
widget_control, event.top, set_uvalue=state, /no_copy
end


pro aps_hdf_browser, filename

widget = { $
            open:       0L, $
            close:      0L, $
            exit:       0L, $
            view_parent:0L, $
            view_mode:  0L, $
            view:       0L $
}

state = { $
        widget:     widget, $
        filename:   "", $
        sdsid:      -1L, $
        view_mode:  2L, $
        fileid:     -1L}
        
base = widget_base( title = 'APS HDF Browser', mbar = mbar, /column, $
                    resource = 'aps_hdf_browser')

default_font = get_font_name(/medium, /bold)
large_font = get_font_name(/large, /italic, /bold)
widget_control, base, default_font = default_font

file           = widget_button( mbar, /menu,               $
                                font = large_font, $
                                value = 'File      ')
state.widget.open           = widget_button( file, value = 'Open . . .')
state.widget.close          = widget_button( file, value = 'Close')
state.widget.exit           = widget_button( file, /separator, value = 'Exit',$
                                              font = large_font)

state.widget.view_parent = widget_base( base, /column)
state.widget.view_mode = widget_droplist(state.widget.view_parent, $
    value=['Vgroups only', 'Vgroups and SDS', 'Vgroups, SDS and attributes'])
widget_control, state.widget.view_mode, set_droplist_select=state.view_mode
if (n_elements(filename) ne 0) then state.filename = filename
aps_open_file, state
new_menu, state
widget_control, base, set_uvalue=state
widget_control, base, /realize
xmanager, 'aps_hdf_browser', base
end
